import json
import time

import requests

from StarRailGPS.utils.resources import resource_path

response = requests.get(
    "https://api-static.mihoyo.com/common/srmap/sr_map/v1/map/tree?map_id=38&app_sn=sr_map&lang=zh-cn")

tree = response.json()['data']['tree']

with open(resource_path('maps/tree.json'), 'w', encoding='utf-8') as f:
    json.dump(tree, f, indent=4, ensure_ascii=False)

name_id = {}

for parent in tree:
    for child in parent['children']:
        map_id = child['id']
        map_name = child['name']

        name_id[map_name] = map_id
        detail_url = 'https://api-static.mihoyo.com/common/srmap/sr_map/v1/map/info?map_id={}&app_sn=sr_map&lang=zh-cn'.format(
            map_id)
        detail_resp = requests.get(detail_url)

        detail_str = detail_resp.json()['data']['info']['detail']

        detail = json.loads(detail_str)
        img_url = detail['slices'][0][0]['url']

        img_resp = requests.get(img_url)
        with open(resource_path('maps/{}.png'.format(map_id)), 'wb') as f:
            f.write(img_resp.content)

        time.sleep(1)

with open(resource_path('maps/name_id.json'), 'w', encoding='utf-8') as f:
    json.dump(name_id, f, indent=4, ensure_ascii=False)



import json
from typing import Tuple

import cv2 as cv
import numpy as np

from StarRailGPS.utils.resources import resource_path

scale = 0.82
minimap_rect = [77, 88, 127, 127]

with open(resource_path('maps/name_id.json'), 'r', encoding='utf-8') as f:
    name_id_map = json.load(f)


def get_mask_from_gray_map(bgra_img):
    b, g, r, a = cv.split(bgra_img)
    gray = b
    mask = (a > 250) & (gray > 80)
    return mask.astype(np.uint8) * 255


def get_mask_from_rgb_min_map(rgb_img):
    img_hsv = cv.cvtColor(rgb_img, cv.COLOR_BGR2HSV)
    h, s, v = cv.split(img_hsv)
    # 筛选白色 H S<10  V 60~90%
    mask1 = (s < 25) * (v > 255 * 0.6) * (v < 255 * 0.9)
    # 筛选蓝色摄像头扫过的白色
    mask2 = (95 < h) * (h < 105) * (0 < s) * (s < 50) * (200 < v) * (v < 240)
    mask = mask1 | mask2
    img_mask = mask.astype(np.uint8) * 255
    return img_mask


def position(screen, map_name=None) -> Tuple[int, int]:
    # template
    min_map = screen[minimap_rect[1]:minimap_rect[1] + minimap_rect[3],
              minimap_rect[0]:minimap_rect[0] + minimap_rect[2]]
    template = get_mask_from_rgb_min_map(min_map)

    # 调整模板的大小到最佳匹配大小
    resized_template = cv.resize(template, None, fx=scale, fy=scale, interpolation=cv.INTER_AREA)

    # map
    map_id = name_id_map[map_name]
    map_gry = cv.imread(resource_path('maps/{}.png'.format(map_id)), cv.IMREAD_UNCHANGED)
    map = get_mask_from_gray_map(map_gry)

    # 进行模板匹配
    res = cv.matchTemplate(map, resized_template, cv.TM_CCORR_NORMED)

    min_val, max_val, min_loc, max_loc = cv.minMaxLoc(res)
    h, w = resized_template.shape[:2]
    x, y = max_loc[0] + w / 2, max_loc[1] + h / 2
    return int(x), int(y)



import cv2

from StarRailGPS.utils.resources import resource_path

# 加载图像
img = cv2.imread(resource_path('test_data/screen_1920_1080.png'))
top_left = (11, 1024)
bottom_right = (152, 1061)

img[top_left[1]:bottom_right[1], top_left[0]:bottom_right[0]] = [0, 0, 0]

cv2.imwrite(resource_path('test_data/screen_1920_1080.png'), img)



from setuptools import setup, find_packages

with open('requirements.txt', 'r', encoding='utf-8') as f:
    requirements = f.readlines()
requirements = [r.strip() for r in requirements if not r.startswith('#')]

# Remove version numbers
requirements = [r.split('==')[0] for r in requirements]

setup(
    name="StarRailGPS",
    version="0.0.1",
    setup_requires=['setuptools_scm'],
    use_scm_version=True,
    packages=find_packages(),
    install_requires=requirements,
    author="furacas",
    author_email="s.furacas@outlook.com",
    description="Honkai: Star Rail GPS",
    long_description=open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/furacas/StarRailGPS",
    classifiers=[
        "Programming Language :: Python :: 3"
    ],
    python_requires='>=3.7',
    package_data={
        'StarRailGPS': ['resources/*'],
    },
)



# -*- coding: utf-8 -*-
import cv2
import numpy as np

from StarRailGPS.gps import get_mask_from_gray_map, get_mask_from_rgb_min_map
from StarRailGPS.utils.resources import resource_path

img = cv2.imread(resource_path('maps/50.png'), cv2.IMREAD_UNCHANGED)
img = get_mask_from_gray_map(img)

screen = cv2.imread(resource_path('test_data/screen_1920_1080.png'))
minimap_rect = [77, 88, 127, 127]  # (x, y, width, height)
template = screen[minimap_rect[1]:minimap_rect[1] + minimap_rect[3], minimap_rect[0]:minimap_rect[0] + minimap_rect[2]]
template = get_mask_from_rgb_min_map(template)

h, w = template.shape[::]
# 创建尺度的列表
scales = np.linspace(0.8, 0.9, 100)

# 对每个尺度进行模板匹配
best_match = None
best_score = -np.inf
max_scale = None
for scale in scales:
    # 调整模板的大小
    resized_template = cv2.resize(template, None, fx=scale, fy=scale, interpolation=cv2.INTER_AREA)
    h, w = resized_template.shape[::-1]

    # 进行模板匹配
    res = cv2.matchTemplate(img, resized_template, cv2.TM_CCORR_NORMED)

    # 找到最好的匹配
    min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
    if max_val > best_score:
        best_score = max_val
        best_match = max_loc, (max_loc[0] + w, max_loc[1] + h)
        max_scale = scale

print(max_scale, best_score)

# 0.82
